class
	P_SRC_BSON_STATIC

inherit
	CE_UTILITIES
	S_STDLIB_STATIC
	S_STDIO_STATIC
	S_STRING_STATIC

feature {ANY} -- public attributes

	bson_malloc_func: CE_STATIC [CE_ROUTINE]
		once ("PROCESS")
			create Result.make (malloc_address)
		end

	bson_realloc_func: CE_STATIC [CE_ROUTINE]
		once ("PROCESS")
			create Result.make (realloc_address)
		end

	bson_free_func: CE_STATIC [CE_ROUTINE]
		once ("PROCESS")
			create Result.make (free_address)
		end

	bson_fprintf: CE_STATIC [CE_ROUTINE]
		once ("PROCESS")
			create Result.make (fprintf_address)
		end

	bson_sprintf: CE_STATIC [CE_ROUTINE]
		once ("PROCESS")
			create Result.make (sprintf_address)
		end

	bson_errprintf: CE_STATIC [CE_ROUTINE]
		once ("PROCESS")
			create Result.make (bson_errprintf2_address)
		end

	oid_fuzz_func: CE_STATIC [CE_ROUTINE]
		once ("PROCESS")
			create Result.make (create {CE_ROUTINE}.make_shared (default_pointer))
		end

	oid_inc_func: CE_STATIC [CE_ROUTINE]
		once ("PROCESS")
			create Result.make (create {CE_ROUTINE}.make_shared (default_pointer))
		end

	err_handler: CE_STATIC [CE_ROUTINE]
		once ("PROCESS")
			create Result.make (create {CE_ROUTINE}.make_shared (default_pointer))
		end

feature {ANY} -- public routines

	bson_create: CE_POINTER [PD_BSON]
		do
			Result := (create {CE_CLASS_POINTER [PD_BSON]}.make_cast ((bson_malloc (((create {PD_BSON}.make).structure_size.to_natural_32).to_integer_32))))
		end

	bson_dispose (a_b: CE_POINTER [PD_BSON])
		do
			bson_free (create {CE_VOID_POINTER}.make_cast (a_b))
		end

	bson_copy (a_out, a_in: CE_POINTER [PD_BSON]): INTEGER_32
		local
			l_return: BOOLEAN
		do
			if a_out.is_void then
				Result := (-1)
				l_return := True
			end
			if not l_return and then (not (a_in.item.finished).to_boolean) then
				Result := (-1)
				l_return := True
			end
			if not l_return then
				a_out.item.init_size2 ((bson_size (a_in)))
				memcpy (create {CE_VOID_POINTER}.make_cast (a_out.item.data), create {CE_VOID_POINTER}.make_cast (a_in.item.data), (bson_size (a_in)).to_natural_32).do_nothing
				a_out.item.finished := 1
				Result := 0
			end
		end

	bson_size (a_b: CE_POINTER [PD_BSON]): INTEGER_32
		local
			l_i: CE_ARRAY [INTEGER_32]
			l_return: BOOLEAN
		do
			create {CE_INTEGER_32_ARRAY} l_i.make (<<1>>)

			if a_b.is_void then
				Result := 0
				l_return := True
			elseif (a_b.item.data).is_void then
				Result := 0
				l_return := True
			end
			if not l_return then
				memcpy (create {CE_VOID_POINTER}.make_cast (l_i), create {CE_VOID_POINTER}.make_cast (a_b.item.data), (4).to_natural_32).do_nothing
				Result := (l_i.item)
			end
		end

	hexbyte (a_hex_1: INTEGER_8): INTEGER_8
		local
			l_pos: INTEGER_32
			l_return: BOOLEAN
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					inspect a_hex_1.to_integer_32 -- switch 1 head
					when 48 then l_pos := -1 + 1
					when 49 then l_pos := -1 + 2
					when 50 then l_pos := -1 + 3
					when 51 then l_pos := -1 + 4
					when 52 then l_pos := -1 + 5
					when 53 then l_pos := -1 + 6
					when 54 then l_pos := -1 + 7
					when 55 then l_pos := -1 + 8
					when 56 then l_pos := -1 + 9
					when 57 then l_pos := -1 + 10
					when 65 then l_pos := -1 + 11
					when 97 then l_pos := -1 + 11
					when 66 then l_pos := -1 + 12
					when 98 then l_pos := -1 + 12
					when 67 then l_pos := -1 + 13
					when 99 then l_pos := -1 + 13
					when 68 then l_pos := -1 + 14
					when 100 then l_pos := -1 + 14
					when 69 then l_pos := -1 + 15
					when 101 then l_pos := -1 + 15
					when 70 then l_pos := -1 + 16
					when 102 then l_pos := -1 + 16
					else l_pos := -1 + 17
					end
				when 1 then -- switch 1, case 48
					Result := (0).to_integer_8
					l_return := True
				when 2 then -- switch 1, case 49
					if not l_return then
						Result := (1).to_integer_8
						l_return := True
					end
				when 3 then -- switch 1, case 50
					if not l_return then
						Result := (2).to_integer_8
						l_return := True
					end
				when 4 then -- switch 1, case 51
					if not l_return then
						Result := (3).to_integer_8
						l_return := True
					end
				when 5 then -- switch 1, case 52
					if not l_return then
						Result := (4).to_integer_8
						l_return := True
					end
				when 6 then -- switch 1, case 53
					if not l_return then
						Result := (5).to_integer_8
						l_return := True
					end
				when 7 then -- switch 1, case 54
					if not l_return then
						Result := (6).to_integer_8
						l_return := True
					end
				when 8 then -- switch 1, case 55
					if not l_return then
						Result := (7).to_integer_8
						l_return := True
					end
				when 9 then -- switch 1, case 56
					if not l_return then
						Result := (8).to_integer_8
						l_return := True
					end
				when 10 then -- switch 1, case 57
					if not l_return then
						Result := (9).to_integer_8
						l_return := True
					end
				when 11 then -- switch 1, case 65 | switch 1, case 97
					if not l_return then
						Result := (10).to_integer_8
						l_return := True
					end
				when 12 then -- switch 1, case 66 | switch 1, case 98
					if not l_return then
						Result := (11).to_integer_8
						l_return := True
					end
				when 13 then -- switch 1, case 67 | switch 1, case 99
					if not l_return then
						Result := (12).to_integer_8
						l_return := True
					end
				when 14 then -- switch 1, case 68 | switch 1, case 100
					if not l_return then
						Result := (13).to_integer_8
						l_return := True
					end
				when 15 then -- switch 1, case 69 | switch 1, case 101
					if not l_return then
						Result := (14).to_integer_8
						l_return := True
					end
				when 16 then -- switch 1, case 70 | switch 1, case 102
					if not l_return then
						Result := (15).to_integer_8
						l_return := True
					end
				when 17 then -- switch 1, default:
					if not l_return then
						Result := (0).to_integer_8
					end
				when 18 then -- switch 1, done:
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	bson_oid_from_string (a_oid: CE_POINTER [PD_BSON_OID_T]; a_str: CE_POINTER [INTEGER_8])
		local
			l_i: INTEGER_32
			l_tmp: INTEGER_8
		do
			l_i := 0
			from
			until
				not (l_i < 12)
			loop
				l_tmp := hexbyte (a_str[2 * l_i])
				a_oid.item.bytes[l_i] := ((((l_tmp.to_integer_32).bit_shift_left (4))).bit_or ((hexbyte (a_str[(2 * l_i + 1)])).to_integer_32)).to_integer_8
				l_i := l_i + 1
			end
		end

	bson_set_oid_fuzz (a_func: CE_ROUTINE)
		do
			oid_fuzz_func.item := a_func
		end

	bson_set_oid_inc (a_func: CE_ROUTINE)
		do
			oid_inc_func.item := a_func
		end

	bson_oid_generated_time (a_oid: CE_POINTER [PD_BSON_OID_T]): INTEGER_32
		local
			l_out: CE_ARRAY [INTEGER_32]
		do
			create {CE_INTEGER_32_ARRAY} l_out.make (<<1>>)

			bson_swap_endian32 (create {CE_VOID_POINTER}.make_cast (l_out), create {CE_VOID_POINTER}.make_cast ((create {CE_INTEGER_32_POINTER}.make_shared (a_oid.c_pointer + (create {PD_BSON_OID_T}.make).offset_of ("ints") + {PLATFORM}.Integer_32_bytes * (0)))))
			Result := (l_out.item)
		end

	bson_iterator_create: CE_POINTER [PD_BSON_ITERATOR]
		do
			Result := (create {CE_CLASS_POINTER [PD_BSON_ITERATOR]}.make_cast ((malloc ((create {PD_BSON_ITERATOR}.make).structure_size.to_natural_32))))
		end

	bson_iterator_dispose (a_i: CE_POINTER [PD_BSON_ITERATOR])
		do
			free (create {CE_VOID_POINTER}.make_cast (a_i))
		end

	bson_append_string_base (a_b: CE_POINTER [PD_BSON]; a_name, a_value: CE_POINTER [INTEGER_8]; a_len: INTEGER_32; a_type: NATURAL_32): INTEGER_32
		local
			l_sl: CE_ARRAY [INTEGER_32]
			l_return: BOOLEAN
		do
			create {CE_INTEGER_32_ARRAY} l_sl.make (<<1>>)

			l_sl.item := a_len + 1
			if ((create {P_SRC_ENCODING_STATIC}).bson_check_string (a_b, a_value, (l_sl.item - 1))) = -1 then
				Result := (-1)
				l_return := True
			end
			if not l_return then
				if (a_b.item.append_estart (a_type.to_integer_32, a_name, (4 + l_sl.item))) = -1 then
					Result := (-1)
					l_return := True
				end
			end
			if not l_return then
				a_b.item.append32 (create {CE_VOID_POINTER}.make_cast (l_sl))
				a_b.item.append (create {CE_VOID_POINTER}.make_cast (a_value), l_sl.item - 1)
				a_b.item.append (create {CE_VOID_POINTER}.make_cast (ce_cstring ("%/0/")), 1)
				Result := 0
			end
		end

	bson_append_symbol (a_b: CE_POINTER [PD_BSON]; a_name, a_value: CE_POINTER [INTEGER_8]): INTEGER_32
		require
			not a_value.is_void
		do
			Result := bson_append_string_base (a_b, a_name, a_value, (strlen (a_value)).to_integer_32, {PC_BSON_TYPE}.Bson_symbol)
		end

	bson_append_code (a_b: CE_POINTER [PD_BSON]; a_name, a_value: CE_POINTER [INTEGER_8]): INTEGER_32
		require
			not a_value.is_void
		do
			Result := bson_append_string_base (a_b, a_name, a_value, (strlen (a_value)).to_integer_32, {PC_BSON_TYPE}.Bson_code)
		end

	bson_append_string_n (a_b: CE_POINTER [PD_BSON]; a_name, a_value: CE_POINTER [INTEGER_8]; a_len: INTEGER_32): INTEGER_32
		do
			Result := bson_append_string_base (a_b, a_name, a_value, a_len, {PC_BSON_TYPE}.Bson_string)
		end

	bson_append_symbol_n (a_b: CE_POINTER [PD_BSON]; a_name, a_value: CE_POINTER [INTEGER_8]; a_len: INTEGER_32): INTEGER_32
		do
			Result := bson_append_string_base (a_b, a_name, a_value, a_len, {PC_BSON_TYPE}.Bson_symbol)
		end

	bson_append_code_n (a_b: CE_POINTER [PD_BSON]; a_name, a_value: CE_POINTER [INTEGER_8]; a_len: INTEGER_32): INTEGER_32
		do
			Result := bson_append_string_base (a_b, a_name, a_value, a_len, {PC_BSON_TYPE}.Bson_code)
		end

	bson_int64_to_double (a_i64: INTEGER_64): REAL_64
		do
			Result := a_i64.to_double
		end

	set_bson_err_handler (a_func: CE_ROUTINE): CE_ROUTINE
		local
			l_old: CE_ROUTINE
		do
			l_old := err_handler.item
			err_handler.item := a_func
			Result := l_old
		end

	bson_free (a_ptr: CE_POINTER [ANY])
		do
			bson_free_func.item.call ([a_ptr])
		end

	bson_malloc (a_size: INTEGER_32): CE_POINTER [ANY]
		local
			l_p: CE_POINTER [ANY]
		do
			l_p := create {CE_VOID_POINTER}.make_cast (bson_malloc_func.item.pointer_item ([a_size.to_natural_32]))
			bson_fatal_msg (((not l_p.is_void)).to_integer, ce_cstring ("malloc() failed"))
			Result := l_p
		end

	bson_realloc (a_ptr: CE_POINTER [ANY]; a_size: INTEGER_32): CE_POINTER [ANY]
		local
			l_p: CE_POINTER [ANY]
		do
			l_p := create {CE_VOID_POINTER}.make_cast (bson_realloc_func.item.pointer_item ([a_ptr, a_size.to_natural_32]))
			bson_fatal_msg (((not l_p.is_void)).to_integer, ce_cstring ("realloc() failed"))
			Result := l_p
		end

	bson_errprintf2 (a_args: TUPLE [a_format: CE_POINTER [INTEGER_8]]): INTEGER_32
		local
			l_ap: CE_VA_LIST
			l_ret: INTEGER_32
		do
			create l_ap.make_shared (default_pointer)

			l_ap := create {CE_VA_LIST}.make (a_args, agent bson_errprintf2)
			l_ret := vfprintf (ce_error_file_pointer, a_args.a_format, l_ap.twin)
			Result := l_ret
		end

	bson_fatal (a_ok: INTEGER_32)
		do
			bson_fatal_msg (a_ok, ce_cstring (""))
		end

	bson_fatal_msg (a_ok: INTEGER_32; a_msg: CE_POINTER [INTEGER_8])
		local
			l_return: BOOLEAN
		do
			if a_ok.to_boolean then
				l_return := True
			end
			if not l_return then
				if (not err_handler.item.is_void) then
					err_handler.item.call ([a_msg])
				end
				bson_errprintf.item.call ([ce_cstring ("error: %%s%N"), a_msg])
				exit (-5)
			end
		end

	bson_swap_endian64 (a_outp, a_inp: CE_POINTER [ANY])
		local
			l_in: CE_POINTER [INTEGER_8]
			l_out: CE_POINTER [INTEGER_8]
		do
			l_in := create {CE_INTEGER_8_POINTER}.make_cast (a_inp)
			l_out := create {CE_INTEGER_8_POINTER}.make_cast (a_outp)
			l_out[0] := l_in[7]
			l_out[1] := l_in[6]
			l_out[2] := l_in[5]
			l_out[3] := l_in[4]
			l_out[4] := l_in[3]
			l_out[5] := l_in[2]
			l_out[6] := l_in[1]
			l_out[7] := l_in[0]
		end

	bson_swap_endian32 (a_outp, a_inp: CE_POINTER [ANY])
		local
			l_in: CE_POINTER [INTEGER_8]
			l_out: CE_POINTER [INTEGER_8]
		do
			l_in := create {CE_INTEGER_8_POINTER}.make_cast (a_inp)
			l_out := create {CE_INTEGER_8_POINTER}.make_cast (a_outp)
			l_out[0] := l_in[3]
			l_out[1] := l_in[2]
			l_out[2] := l_in[1]
			l_out[3] := l_in[0]
		end

feature {ANY} -- routine addresses

	bson_errprintf2_address: CE_ROUTINE
		once ("PROCESS")
			create Result.make_agent (agent bson_errprintf2)
		end

end
