class
	PD_MONGO_WRITE_CONCERN

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			make_shared,
			structure_size,
			offset_of,
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			internal_c_pointer := internal_c_pointer.memory_alloc (structure_size)
			is_allocated_memory := True
		end

	make_shared (a_c_pointer: POINTER)
		do
			internal_c_pointer := a_c_pointer
		end

	make_initialized (a_data: TUPLE [
				a_w: INTEGER_32
				a_wtimeout: INTEGER_32
				a_j: INTEGER_32
				a_fsync: INTEGER_32
				a_mode: CE_POINTER [INTEGER_8]
				a_cmd: CE_POINTER [PD_BSON]
			])
		require else
			a_data /= Void
		do
			make
			Current.w := a_data.a_w
			Current.wtimeout := a_data.a_wtimeout
			Current.j := a_data.a_j
			Current.fsync := a_data.a_fsync
			Current.mode := a_data.a_mode
			Current.cmd := a_data.a_cmd
		end

feature {ANY} -- fields

	w: INTEGER_32 assign set__w
		do
			Result := c__w (internal_c_pointer)
		end

	wtimeout: INTEGER_32 assign set__wtimeout
		do
			Result := c__wtimeout (internal_c_pointer)
		end

	j: INTEGER_32 assign set__j
		do
			Result := c__j (internal_c_pointer)
		end

	fsync: INTEGER_32 assign set__fsync
		do
			Result := c__fsync (internal_c_pointer)
		end

	mode: CE_POINTER [INTEGER_8] assign set__mode
		do
			create {CE_INTEGER_8_POINTER} Result.make_shared (c__mode (internal_c_pointer))
		end

	cmd: CE_POINTER [PD_BSON] assign set__cmd
		do
			create {CE_CLASS_POINTER [PD_BSON]} Result.make_shared (c__cmd (internal_c_pointer))
		end

feature {ANY} -- field setters

	set__w (a_w: INTEGER_32)
		do
			c__set_w (internal_c_pointer, a_w)
		end

	set__wtimeout (a_wtimeout: INTEGER_32)
		do
			c__set_wtimeout (internal_c_pointer, a_wtimeout)
		end

	set__j (a_j: INTEGER_32)
		do
			c__set_j (internal_c_pointer, a_j)
		end

	set__fsync (a_fsync: INTEGER_32)
		do
			c__set_fsync (internal_c_pointer, a_fsync)
		end

	set__mode (a_mode: CE_POINTER [INTEGER_8])
		do
			c__set_mode (internal_c_pointer, a_mode.c_pointer)
		end

	set__cmd (a_cmd: CE_POINTER [PD_BSON])
		do
			c__set_cmd (internal_c_pointer, a_cmd.c_pointer)
		end

feature {ANY} -- public routines

	finish: INTEGER_32
		local
			l_command: CE_POINTER [PD_BSON]
			l_return: BOOLEAN
		do
			if (not cmd.is_void) then
				cmd.item.destroy
				l_command := cmd
			else
				l_command := create {CE_CLASS_POINTER [PD_BSON]}.make_cast (((create {P_SRC_BSON_STATIC}).bson_malloc (((create {PD_BSON}.make).structure_size.to_natural_32).to_integer_32)))
			end
			if l_command.is_void then
				Result := (-1)
				l_return := True
			end
			if not l_return then
				l_command.item.init
				l_command.item.append_int (ce_cstring ("getlasterror"), 1).do_nothing
				if (not mode.is_void) then
					l_command.item.append_string (ce_cstring ("w"), mode).do_nothing
				elseif w.to_boolean then
					l_command.item.append_int (ce_cstring ("w"), w).do_nothing
				end
				if wtimeout.to_boolean then
					l_command.item.append_int (ce_cstring ("wtimeout"), wtimeout).do_nothing
				end
				if j.to_boolean then
					l_command.item.append_int (ce_cstring ("j"), j).do_nothing
				end
				if fsync.to_boolean then
					l_command.item.append_int (ce_cstring ("fsync"), fsync).do_nothing
				end
				l_command.item.finish.do_nothing
				Current.cmd := l_command
				Result := 0
			end
		end

feature {ANY}

	structure_size: INTEGER_32
		external
			"C inline use %"libmongo.h%""
		alias
			"return sizeof(struct mongo_write_concern);"
		end

	structure_alignment: INTEGER_32
		external
			"C inline use %"libmongo.h%""
		alias
			"return __alignof__(struct mongo_write_concern);"
		end

	offset_of (a_field_name: STRING_8): INTEGER_32
		local
			l_c_str: C_STRING
		do
			create l_c_str.make (a_field_name)
			Result := c__offset_of (l_c_str.item)
		end

feature {ANY}

	copy (a_other: like Current)
		do
			make
			internal_c_pointer.memory_copy (a_other.c_pointer, structure_size)
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result := internal_c_pointer.memory_compare (a_other.c_pointer, structure_size)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		do
			-- dummy for CE_CLASS invariant
			create Result.make_struct (<<>>)
			Result.alignment := structure_alignment
		end

	c__w (a_c_pointer: POINTER): INTEGER_32
		external
			"C inline use %"libmongo.h%""
		alias
			"return ((struct mongo_write_concern *)$a_c_pointer)->w;"
		end

	c__wtimeout (a_c_pointer: POINTER): INTEGER_32
		external
			"C inline use %"libmongo.h%""
		alias
			"return ((struct mongo_write_concern *)$a_c_pointer)->wtimeout;"
		end

	c__j (a_c_pointer: POINTER): INTEGER_32
		external
			"C inline use %"libmongo.h%""
		alias
			"return ((struct mongo_write_concern *)$a_c_pointer)->j;"
		end

	c__fsync (a_c_pointer: POINTER): INTEGER_32
		external
			"C inline use %"libmongo.h%""
		alias
			"return ((struct mongo_write_concern *)$a_c_pointer)->fsync;"
		end

	c__mode (a_c_pointer: POINTER): POINTER
		external
			"C inline use %"libmongo.h%""
		alias
			"return (EIF_POINTER)((struct mongo_write_concern *)$a_c_pointer)->mode;"
		end

	c__cmd (a_c_pointer: POINTER): POINTER
		external
			"C inline use %"libmongo.h%""
		alias
			"return (EIF_POINTER)((struct mongo_write_concern *)$a_c_pointer)->cmd;"
		end

	c__set_w (a_c_pointer: POINTER; a_w: INTEGER_32)
		external
			"C inline use %"libmongo.h%""
		alias
			"((struct mongo_write_concern *)$a_c_pointer)->w = $a_w;"
		end

	c__set_wtimeout (a_c_pointer: POINTER; a_wtimeout: INTEGER_32)
		external
			"C inline use %"libmongo.h%""
		alias
			"((struct mongo_write_concern *)$a_c_pointer)->wtimeout = $a_wtimeout;"
		end

	c__set_j (a_c_pointer: POINTER; a_j: INTEGER_32)
		external
			"C inline use %"libmongo.h%""
		alias
			"((struct mongo_write_concern *)$a_c_pointer)->j = $a_j;"
		end

	c__set_fsync (a_c_pointer: POINTER; a_fsync: INTEGER_32)
		external
			"C inline use %"libmongo.h%""
		alias
			"((struct mongo_write_concern *)$a_c_pointer)->fsync = $a_fsync;"
		end

	c__set_mode (a_c_pointer: POINTER; a_mode: POINTER)
		external
			"C inline use %"libmongo.h%""
		alias
			"((struct mongo_write_concern *)$a_c_pointer)->mode = $a_mode;"
		end

	c__set_cmd (a_c_pointer: POINTER; a_cmd: POINTER)
		external
			"C inline use %"libmongo.h%""
		alias
			"((struct mongo_write_concern *)$a_c_pointer)->cmd = $a_cmd;"
		end

	c__offset_of (a_field_name: POINTER): INTEGER_32
		external
			"C inline use %"libmongo.h%", <string.h>, <stddef.h>, <stdio.h>"
		alias
			"[
				if(strcmp($a_field_name, "w") == 0) return offsetof(struct mongo_write_concern, w);
				if(strcmp($a_field_name, "wtimeout") == 0) return offsetof(struct mongo_write_concern, wtimeout);
				if(strcmp($a_field_name, "j") == 0) return offsetof(struct mongo_write_concern, j);
				if(strcmp($a_field_name, "fsync") == 0) return offsetof(struct mongo_write_concern, fsync);
				if(strcmp($a_field_name, "mode") == 0) return offsetof(struct mongo_write_concern, mode);
				if(strcmp($a_field_name, "cmd") == 0) return offsetof(struct mongo_write_concern, cmd);
				fprintf(stderr, "struct mongo_write_concern.%s not found\n", (char *)$a_field_name);
				return -1;
			]"
		end

end
