class
	SD_WSADATA

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			make_shared,
			structure_size,
			offset_of,
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			internal_c_pointer := internal_c_pointer.memory_alloc (structure_size)
			is_allocated_memory := True
		end

	make_shared (a_c_pointer: POINTER)
		do
			internal_c_pointer := a_c_pointer
		end

	make_initialized (a_data: TUPLE [
				a_w_version: NATURAL_16
				a_w_high_version: NATURAL_16
				a_sz_description: CE_ARRAY [INTEGER_8]
				a_sz_system_status: CE_ARRAY [INTEGER_8]
				a_i_max_sockets: NATURAL_16
				a_i_max_udp_dg: NATURAL_16
				a_lp_vendor_info: CE_POINTER [INTEGER_8]
			])
		require else
			a_data /= Void
		do
			make
			Current.w_version := a_data.a_w_version
			Current.w_high_version := a_data.a_w_high_version
			sz_description.c_pointer.memory_copy (a_data.a_sz_description.c_pointer, a_data.a_sz_description.total_count * a_data.a_sz_description.element_size)
			sz_system_status.c_pointer.memory_copy (a_data.a_sz_system_status.c_pointer, a_data.a_sz_system_status.total_count * a_data.a_sz_system_status.element_size)
			Current.i_max_sockets := a_data.a_i_max_sockets
			Current.i_max_udp_dg := a_data.a_i_max_udp_dg
			Current.lp_vendor_info := a_data.a_lp_vendor_info
		end

feature {ANY} -- fields

	w_version: NATURAL_16 assign set__w_version
		do
			Result := c__w_version (internal_c_pointer)
		end

	w_high_version: NATURAL_16 assign set__w_high_version
		do
			Result := c__w_high_version (internal_c_pointer)
		end

	sz_description: CE_ARRAY [INTEGER_8]
		once ("OBJECT")
			create {CE_INTEGER_8_ARRAY} Result.make_shared (c__sz_description (internal_c_pointer), <<257>>)
		end

	sz_system_status: CE_ARRAY [INTEGER_8]
		once ("OBJECT")
			create {CE_INTEGER_8_ARRAY} Result.make_shared (c__sz_system_status (internal_c_pointer), <<129>>)
		end

	i_max_sockets: NATURAL_16 assign set__i_max_sockets
		do
			Result := c__i_max_sockets (internal_c_pointer)
		end

	i_max_udp_dg: NATURAL_16 assign set__i_max_udp_dg
		do
			Result := c__i_max_udp_dg (internal_c_pointer)
		end

	lp_vendor_info: CE_POINTER [INTEGER_8] assign set__lp_vendor_info
		do
			create {CE_INTEGER_8_POINTER} Result.make_shared (c__lp_vendor_info (internal_c_pointer))
		end

feature {ANY} -- field setters

	set__w_version (a_w_version: NATURAL_16)
		do
			c__set_w_version (internal_c_pointer, a_w_version)
		end

	set__w_high_version (a_w_high_version: NATURAL_16)
		do
			c__set_w_high_version (internal_c_pointer, a_w_high_version)
		end

	set__i_max_sockets (a_i_max_sockets: NATURAL_16)
		do
			c__set_i_max_sockets (internal_c_pointer, a_i_max_sockets)
		end

	set__i_max_udp_dg (a_i_max_udp_dg: NATURAL_16)
		do
			c__set_i_max_udp_dg (internal_c_pointer, a_i_max_udp_dg)
		end

	set__lp_vendor_info (a_lp_vendor_info: CE_POINTER [INTEGER_8])
		do
			c__set_lp_vendor_info (internal_c_pointer, a_lp_vendor_info.c_pointer)
		end

feature {ANY}

	structure_size: INTEGER_32
		external
			"C inline use %"libmongo.h%""
		alias
			"return sizeof(struct WSAData);"
		end

	structure_alignment: INTEGER_32
		external
			"C inline use %"libmongo.h%""
		alias
			"return __alignof__(struct WSAData);"
		end

	offset_of (a_field_name: STRING_8): INTEGER_32
		local
			l_c_str: C_STRING
		do
			create l_c_str.make (a_field_name)
			Result := c__offset_of (l_c_str.item)
		end

feature {ANY}

	copy (a_other: like Current)
		do
			make
			internal_c_pointer.memory_copy (a_other.c_pointer, structure_size)
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result := internal_c_pointer.memory_compare (a_other.c_pointer, structure_size)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		do
			-- dummy for CE_CLASS invariant
			create Result.make_struct (<<>>)
			Result.alignment := structure_alignment
		end

	c__w_version (a_c_pointer: POINTER): NATURAL_16
		external
			"C inline use %"libmongo.h%""
		alias
			"return ((struct WSAData *)$a_c_pointer)->wVersion;"
		end

	c__w_high_version (a_c_pointer: POINTER): NATURAL_16
		external
			"C inline use %"libmongo.h%""
		alias
			"return ((struct WSAData *)$a_c_pointer)->wHighVersion;"
		end

	c__sz_description (a_c_pointer: POINTER): POINTER
		external
			"C inline use %"libmongo.h%""
		alias
			"return (EIF_POINTER)((struct WSAData *)$a_c_pointer)->szDescription;"
		end

	c__sz_system_status (a_c_pointer: POINTER): POINTER
		external
			"C inline use %"libmongo.h%""
		alias
			"return (EIF_POINTER)((struct WSAData *)$a_c_pointer)->szSystemStatus;"
		end

	c__i_max_sockets (a_c_pointer: POINTER): NATURAL_16
		external
			"C inline use %"libmongo.h%""
		alias
			"return ((struct WSAData *)$a_c_pointer)->iMaxSockets;"
		end

	c__i_max_udp_dg (a_c_pointer: POINTER): NATURAL_16
		external
			"C inline use %"libmongo.h%""
		alias
			"return ((struct WSAData *)$a_c_pointer)->iMaxUdpDg;"
		end

	c__lp_vendor_info (a_c_pointer: POINTER): POINTER
		external
			"C inline use %"libmongo.h%""
		alias
			"return (EIF_POINTER)((struct WSAData *)$a_c_pointer)->lpVendorInfo;"
		end

	c__set_w_version (a_c_pointer: POINTER; a_w_version: NATURAL_16)
		external
			"C inline use %"libmongo.h%""
		alias
			"((struct WSAData *)$a_c_pointer)->wVersion = $a_w_version;"
		end

	c__set_w_high_version (a_c_pointer: POINTER; a_w_high_version: NATURAL_16)
		external
			"C inline use %"libmongo.h%""
		alias
			"((struct WSAData *)$a_c_pointer)->wHighVersion = $a_w_high_version;"
		end

	c__set_i_max_sockets (a_c_pointer: POINTER; a_i_max_sockets: NATURAL_16)
		external
			"C inline use %"libmongo.h%""
		alias
			"((struct WSAData *)$a_c_pointer)->iMaxSockets = $a_i_max_sockets;"
		end

	c__set_i_max_udp_dg (a_c_pointer: POINTER; a_i_max_udp_dg: NATURAL_16)
		external
			"C inline use %"libmongo.h%""
		alias
			"((struct WSAData *)$a_c_pointer)->iMaxUdpDg = $a_i_max_udp_dg;"
		end

	c__set_lp_vendor_info (a_c_pointer: POINTER; a_lp_vendor_info: POINTER)
		external
			"C inline use %"libmongo.h%""
		alias
			"((struct WSAData *)$a_c_pointer)->lpVendorInfo = $a_lp_vendor_info;"
		end

	c__offset_of (a_field_name: POINTER): INTEGER_32
		external
			"C inline use %"libmongo.h%", <string.h>, <stddef.h>, <stdio.h>"
		alias
			"[
				if(strcmp($a_field_name, "w_version") == 0) return offsetof(struct WSAData, wVersion);
				if(strcmp($a_field_name, "w_high_version") == 0) return offsetof(struct WSAData, wHighVersion);
				if(strcmp($a_field_name, "sz_description") == 0) return offsetof(struct WSAData, szDescription);
				if(strcmp($a_field_name, "sz_system_status") == 0) return offsetof(struct WSAData, szSystemStatus);
				if(strcmp($a_field_name, "i_max_sockets") == 0) return offsetof(struct WSAData, iMaxSockets);
				if(strcmp($a_field_name, "i_max_udp_dg") == 0) return offsetof(struct WSAData, iMaxUdpDg);
				if(strcmp($a_field_name, "lp_vendor_info") == 0) return offsetof(struct WSAData, lpVendorInfo);
				fprintf(stderr, "struct WSAData.%s not found\n", (char *)$a_field_name);
				return -1;
			]"
		end

end
